% *************************************************************************
% 
%                      iGem Modelling - 2016
%                     Delft University of Technology
% 
% *************************************************************************
% 
%               Title
% 
%                    File: Laser threshold
%                           Model:The dynamics of GFP excitation after a pulsed laser 
% 
%             
% *************************************************************************
%
function [C,maxima_photons,tt,y]=Laser_threshold
global GFPt QY ii Kstim Ksp Kabs Kmirror;
%%
% This function simulates the amount of excited GFP molecules in a lasing 
% system directly after a pulse assuming all the fluorophores are in the 
% excited state. s

%% Nature constants:
Na          = 6.022e23;                 % [N/mol]           Avogadro Number 
c           = 2.99792458e8;             % [m/s]             speed of light 

%% system size
    % System size for a Single cell
    l           = 1e-6;                 % [m]               Cell diameter, assuming spherical cell of 1um  (also characteristic path length)
    V           = 4/3*pi*(l/2)^3;       % [m^3]             Assuming spherical cell
    savename = ['Laser_threshold_Singlecell_dia_',num2str(l*10^6),'um_R02711_1016'];

    % System size for GFP between 2 mirrors
%     Beamdia     = 0.8e-3;             % [m]               Beam Diameter
%     l           = 7e-3;               % [m]               Space between two mirrors (also characteristic path length)
%     V           = l*(Beamdia/2)^2*pi; % [m^3]             Eluminated volume (spacing d* beam diameter 10mm)
%     savename = ['Laser_threshold_2mirrors_l_',num2str(l*10^3),'mm'];

%% Characteristics of fluorophore (GFP):
t_sp        = 2.71e-9;                  % [s]               Lifetime GFP 
sigma_a     = 2.03e-20;                 % [m^2/N]           GFP absorption cross section = Stimulated emission cross section 
QY          = 0.6;                      %                   Quantum yield
                    
%% System characteristics
n           = 100;                      %                   Number of points taken 
C           = logspace(-5,4,n);         % [Molair]          Molaire concentration of GFP
GFPt        = C*10^3*V*Na;              % [N]               Total number of GFP molecules in the system

%% Rates
R           = 0.2711;                   %                   Reflectivity mirror              
Kmirror     = c / (2 * l) * log((R)^2); % [s^-1]            Mirror loss rate
Ksp         = 1 / t_sp;                 % [s^-1]            Spontaneous relaxation rate
Kstim       = sigma_a * c / V;          % [s^-1 N^-1]       Stimulated emission rate
alpha       = 15.2;                     % [m^-1]          Absorption coefficient cells
Kabs        = -alpha * c;               % [s^-1]            Absorption rate

%% Solving the differential equation
options1        = odeset('InitialStep',1e-15);
% ty            = cell(n,2)    % Saving the maximal photon numbers   
maxima_photons  = zeros(length(GFPt),2);
%%
tf              = 1.5e-9;               % [s]               Stop time
tspan           = [0, tf];              % [s]               Time interval to solve ODE
for ii = 1:length(GFPt)
   ii
   [tt,y]               = ode45(@dp1,tspan,[GFPt(ii),0],options1);
%    ty{ii,1}             = sparse(tt);
%    ty{ii,2}             = sparse(y); 
   [maxy,i]             = max(y(:,2));
   maxima_photons(ii,:) = [maxy,tt(i)];
end
%% Plotting 
figure(1)
hold on 
plot(C*10^6,-maxima_photons(:,1)*Kmirror,'-o')
title('Maximal output intensity')
xlabel('Concentration [\muM]')
ylabel('number of photons [N/s]')

%% Saving variables
save(savename,'-v7.3')
% save('01_20muMtime.mat','-v7.3')
% save([savename,'ty.mat'], 'ty')
end
%%
function dydt = dp1(t,y)
global GFPt QY ii Kstim Ksp Kabs Kmirror; 
dydt=zeros(2,1);
% % the differential equations
% Spontaneous +Stimulated +mirrors
% Number of GFP in excited state
dydt(1)             = -(Ksp + Kstim * y(2)) * y(1);                                 % dGFP1/dt 
if y(1)>GFPt(ii)
    % GFP1 is not aloud to be larger than the total number of GFP molecules
    y(1)            = GFPt(ii);
end
% Number of photons N1
dydt(2)             = (QY * Ksp + Kstim * y(2)) * y(1) + (Kmirror + Kabs) * y(2);   % dN1/dt
end
